unit XDBGridUnit3;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBCtrls, DBTables, Grids, XDBGrids, ExtCtrls, jpeg, ImgList, StdCtrls;

type
  TXDBGridForm3 = class(TForm)
    BioLifeDataSource: TDataSource;
      BioLife: TTable;
        BioLifeSpeciesNo: TFloatField;
        BioLifeCategory: TStringField;
        BioLifeCommon_Name: TStringField;
        BioLifeSpeciesName: TStringField;
        BioLifeLengthcm: TFloatField;
        BioLifeLength_In: TFloatField;
        BioLifeNotes: TMemoField;
        BioLifeGraphic: TGraphicField;
    BioLifePanel: TPanel;
      BioLifeDBGrid: TXDBGrid;
    CommentsPanel: TPanel;
      ImageListBMP: TImageList;
      ImageListICO: TImageList;
      ImagesGroupBox: TGroupBox;
        ImageJPG1: TImage;
        ImageJPG2: TImage;
        ImageJPG3: TImage;
        ImageJPG4: TImage;
        ImageWMF1: TImage;
        ImageWMF2: TImage;
        ImageWMF3: TImage;
        ImageWMF4: TImage;
      ImageDrawRadioGroup: TRadioGroup;
      AlignmentRadioGroup: TRadioGroup;
      VAlignmentRadioGroup: TRadioGroup;
      OptionsGroupBox: TGroupBox;
        TransparentCheckBox: TCheckBox;
        WallpaperCheckBox: TCheckBox;
      PictureRadioGroup: TRadioGroup;
      WallpapersGroupBox: TGroupBox;
        ImageJPG5: TImage;
        ImageJPG6: TImage;
        ImageJPG7: TImage;
        ImageJPG8: TImage;
        ImageJPG9: TImage;
        ImageJPG10: TImage;
        ImageJPG11: TImage;
        ImageJPG12: TImage;
        ImageJPG13: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BioLifeCategoryGetText(Sender: TField; var Text: String;
      DisplayText: Boolean);
    procedure BioLifeLengthcmGetText(Sender: TField; var Text: String;
      DisplayText: Boolean);
    procedure BioLifeDBGridPaintColumnCell(Sender: TObject;
      const Rect: TRect; DataCol: Integer; Column: TXColumn; Highlight: Boolean;
      var Color: TColor; Font: TFont; var Image: TPersistent);
    procedure OptionsChanged(Sender: TObject);
    procedure ImageJPGClick(Sender: TObject);
    procedure ImageWMFClick(Sender: TObject);
    procedure PictureClick(Sender: TObject);
    procedure WallpaperClick(Sender: TObject);
  private
    Wallpaper: TImage;
    Bitmap: TBitmap;
    Icon: TIcon;
  end;

var
  XDBGridForm3: TXDBGridForm3;

implementation

uses Math;

{$R *.DFM}

procedure TXDBGridForm3.FormCreate(Sender: TObject);
begin
  Bitmap := TBitmap.Create;
  Icon := TIcon.Create;
end;

procedure TXDBGridForm3.FormDestroy(Sender: TObject);
begin
  Bitmap.Free;
  Icon.Free;
end;

procedure TXDBGridForm3.FormResize(Sender: TObject);
var
  I, AWidth: Integer;
begin
  AWidth := IndicatorWidth;
  for I := 0 to BioLifeDBGrid.Columns.Count-1 do
    Inc(AWidth, BioLifeDBGrid.Columns[I].Width + 1);
  if AWidth <> BioLifeDBGrid.ClientWidth then
  with BioLifeDBGrid.ColumnByName('Notes') do
    Width := BioLifeDBGrid.ClientWidth - AWidth + Width;
end;

procedure TXDBGridForm3.BioLifeCategoryGetText(Sender: TField; var Text: String;
  DisplayText: Boolean);
const
  Str = '%s'#13'%s'#13'(%s)';
begin
  Text := Format(Str, [BioLifeCategory.AsString, BioLifeCommon_Name.AsString,
    BioLifeSpeciesName.AsString]);
end;

procedure TXDBGridForm3.BioLifeLengthcmGetText(Sender: TField; var Text: String;
  DisplayText: Boolean);
const
  Str = '%f "'#13'(%d cm)';
begin
  Text := Format(Str, [BioLifeLength_In.AsFloat, BioLifeLengthcm.AsInteger]);
end;

procedure TXDBGridForm3.BioLifeDBGridPaintColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TXColumn; Highlight: Boolean;
  var Color: TColor; Font: TFont; var Image: TPersistent);
var
  ImageIndex: Integer;
begin
  if Wallpaper <> nil then
  begin
    Image := Wallpaper.Picture;
    if not Highlight and (StrToInt(Copy(Wallpaper.Name, 9, 2)) >= 10) then
      Font.Color := clYellow;
  end else
  if Column.FieldName = 'Length (cm)' then
  begin
    ImageIndex := Max(Min((BioLifeLengthCm.AsInteger-1) div 50, 3), 0);
    case PictureRadioGroup.ItemIndex of
      1: begin
           ImageListBMP.GetBitmap(ImageIndex, Bitmap);
           Image := Bitmap;
         end;
      2: begin
           ImageListICO.GetIcon(ImageIndex, Icon);
           Image := Icon;
         end;
      3: Image := TImage(FindComponent('ImageJPG'+IntToStr(ImageIndex+1))).Picture;
      4: Image := TImage(FindComponent('ImageWMF'+IntToStr(ImageIndex+1))).Picture;
    end;
  end;
end;

procedure TXDBGridForm3.OptionsChanged(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to BioLifeDBGrid.Columns.Count-1 do
  with BioLifeDBGrid.Columns[I] do
    if (Self.Wallpaper <> nil) or (FieldName = 'Length (cm)') then
    begin
      ImageDraw := TImageDraw(ImageDrawRadioGroup.ItemIndex);
      Alignment := TAlignment(AlignmentRadioGroup.ItemIndex);
      VAlignment := TVAlignment(VAlignmentRadioGroup.ItemIndex);
      Transparent := TransparentCheckBox.Checked;
      Wallpaper := WallpaperCheckBox.Checked;
    end;
end;

procedure TXDBGridForm3.ImageJPGClick(Sender: TObject);
begin
  PictureRadioGroup.ItemIndex := 3;
end;

procedure TXDBGridForm3.ImageWMFClick(Sender: TObject);
begin
  PictureRadioGroup.ItemIndex := 4;
end;

procedure TXDBGridForm3.PictureClick(Sender: TObject);
var
  I: Integer;
begin
  if Wallpaper <> nil then
  begin {Restore default columns layout}
    for I := 0 to BioLifeDBGrid.Columns.Count-1 do
    with BioLifeDBGrid.Columns[I] do
    begin
      ImageDraw := idCenter;
      Alignment := taCenter;
      VAlignment := tvCenter;
      Transparent := False;
      Wallpaper := False;
    end;
    BioLifeDBGrid.ColumnByName('Notes').Alignment := taLeftJustify;
    BioLifeDBGrid.ColumnByName('Graphic').ImageDraw := idStretch;
    Wallpaper := nil;
    ImageDrawRadioGroup.ItemIndex := 2; {idCenter}
    WallpaperCheckBox.Checked := False;
  end;
  BioLifeDBGrid.Refresh;
end;

procedure TXDBGridForm3.WallpaperClick(Sender: TObject);
begin
  if Wallpaper = nil then
  begin {Setup wallpaper columns layout}
    PictureRadioGroup.ItemIndex := 0;
    ImageDrawRadioGroup.ItemIndex := 5; {TileGrid}
    WallpaperCheckBox.Checked := True;
    Wallpaper := TImage(Sender);
    OptionsChanged(nil);
  end
  else Wallpaper := TImage(Sender);
  BioLifeDBGrid.Refresh;
end;

end.
 